@@ -12,13 +12,14 @@ use p2p::{
     service::{SessionType, TargetProtocol},
     traits::ServiceProtocol,
     utils::{extract_peer_id, is_reachable, multiaddr_to_socketaddr},
-    ProtocolId, SessionId,
+    SessionId,
 };
 
 mod protocol;
 
 use crate::{NetworkState, PeerIdentifyInfo, SupportProtocols};
 use ckb_types::{packed, prelude::*};
+use std::sync::atomic::Ordering;
 
 use protocol::IdentifyMessage;
 
@@ -59,9 +60,9 @@ impl MisbehaveResult {
 /// The trait to communicate with underlying peer storage
 pub trait Callback: Clone + Send {
     // Register open protocol
-    fn register(&self, id: SessionId, pid: ProtocolId, version: &str);
+    fn register(&self, context: &ProtocolContextMutRef, version: &str);
     // remove registered identify protocol
-    fn unregister(&self, id: SessionId, pid: ProtocolId);
+    fn unregister(&self, context: &ProtocolContextMutRef);
     /// Received custom message
     fn received_identify(
         &mut self,
@@ -103,6 +104,12 @@ impl<T: Callback> IdentifyProtocol<T> {
         }
     }
 
+    #[cfg(test)]
+    pub fn global_ip_only(mut self, only: bool) -> Self {
+        self.global_ip_only = only;
+        self
+    }
+
     fn check_duplicate(&mut self, context: &mut ProtocolContextMutRef) -> MisbehaveResult {
         let session = context.session;
         let info = self
@@ -229,8 +236,7 @@ impl<T: Callback> ServiceProtocol for IdentifyProtocol<T> {
             return;
         }
 
-        self.callback
-            .register(session.id, context.proto_id, version);
+        self.callback.register(&context, version);
 
         let remote_info = RemoteInfo::new(session.clone(), Duration::from_secs(DEFAULT_TIMEOUT));
         trace!("IdentifyProtocol sconnected from {:?}", remote_info.peer_id);
@@ -261,8 +267,7 @@ impl<T: Callback> ServiceProtocol for IdentifyProtocol<T> {
                 .remove(&context.session.id)
                 .expect("RemoteInfo must exists");
             trace!("IdentifyProtocol disconnected from {:?}", info.peer_id);
-            self.callback
-                .unregister(context.session.id, context.proto_id)
+            self.callback.unregister(&context)
         }
     }
 
@@ -361,20 +366,52 @@ impl IdentifyCallback {
 }
 
 impl Callback for IdentifyCallback {
-    fn register(&self, id: SessionId, pid: ProtocolId, version: &str) {
+    fn register(&self, context: &ProtocolContextMutRef, version: &str) {
         self.network_state.with_peer_registry_mut(|reg| {
-            reg.get_peer_mut(id).map(|peer| {
-                peer.protocols.insert(pid, version.to_owned());
+            reg.get_peer_mut(context.session.id).map(|peer| {
+                peer.protocols.insert(context.proto_id, version.to_owned());
             })
         });
+        if self.network_state.ckb2021.load(Ordering::SeqCst) && version != "2" {
+            self.network_state
+                .peer_store
+                .lock()
+                .mut_addr_manager()
+                .remove(&context.session.address);
+        } else if context.session.ty.is_outbound() {
+            // why don't set inbound here?
+            // because inbound address can't feeler during staying connected
+            // and if set it to peer store, it will be broadcast to the entire network,
+            // but this is an unverified address
+            self.network_state.with_peer_store_mut(|peer_store| {
+                peer_store.add_outbound_addr(context.session.address.clone());
+            });
+        }
     }
 
-    fn unregister(&self, id: SessionId, pid: ProtocolId) {
-        self.network_state.with_peer_registry_mut(|reg| {
-            let _ = reg.get_peer_mut(id).map(|peer| {
-                peer.protocols.remove(&pid);
+    fn unregister(&self, context: &ProtocolContextMutRef) {
+        let version = self
+            .network_state
+            .with_peer_registry_mut(|reg| {
+                reg.get_peer_mut(context.session.id)
+                    .map(|peer| peer.protocols.remove(&context.proto_id))
+            })
+            .flatten()
+            .map(|version| version != "2")
+            .unwrap_or_default();
+
+        if self.network_state.ckb2021.load(Ordering::SeqCst) && version {
+        } else if context.session.ty.is_outbound() {
+            // Due to the filtering strategy of the peer store, if the node is
+            // disconnected after a long connection is maintained for more than seven days,
+            // it is possible that the node will be accidentally evicted, so it is necessary
+            // to reset the information of the node when disconnected.
+            self.network_state.with_peer_store_mut(|peer_store| {
+                if !peer_store.is_addr_banned(&context.session.address) {
+                    peer_store.add_outbound_addr(context.session.address.clone());
+                }
             });
-        });
+        }
     }
 
     fn identify(&mut self) -> &[u8] {
@@ -419,11 +456,20 @@ impl Callback for IdentifyCallback {
                     } else if flags.contains(self.identify.flags) {
                         registry_client_version(client_version);
 
+                        let ckb2021 = self
+                            .network_state
+                            .ckb2021
+                            .load(std::sync::atomic::Ordering::SeqCst);
                         // The remote end can support all local protocols.
                         let _ = context.open_protocols(
                             context.session.id,
-                            TargetProtocol::Filter(Box::new(|id| {
-                                id != &SupportProtocols::Feeler.protocol_id()
+                            TargetProtocol::Filter(Box::new(move |id| {
+                                if ckb2021 {
+                                    id != &SupportProtocols::Feeler.protocol_id()
+                                        && id != &SupportProtocols::Relay.protocol_id()
+                                } else {
+                                    id != &SupportProtocols::Feeler.protocol_id()
+                                }
                             })),
                         );
                     } else {
